use strict;
use warnings;

# this test was generated with Dist::Zilla::Plugin::Test::EOL 0.17

use Test::More 0.88;
use Test::EOL;

my @files = (
    'lib/Test/LWP/UserAgent.pm',
    't/00-report-prereqs.dd',
    't/00-report-prereqs.t',
    't/01-basic.t',
    't/02-overload.t',
    't/03-handlers.t',
    't/04-psgi.t',
    't/05-exceptions.t',
    't/06-network-fallback.t',
    't/07-mask-response.t',
    't/08-isa-coderef.t',
    't/09-dispatch-to-request-method.t',
    't/10-request-args-network.t',
    't/11-request-args-internal.t',
    't/12-chained-method-call.t',
    't/50-examples-application_client_test.t',
    't/51-call_psgi.t'
);

eol_unix_ok($_, { trailing_whitespace => 1 }) foreach @files;
done_testing;
